const fs = require('fs-extra');

class AsyncFileSystem {
    /**
     * Determine if the given path exists 
     * 
     * @param   string path
     * @returns boolean
     */
    exists(path) {
        return fs.exists(path);
    }

    /**
     * Load the given json file path
     * 
     * @param  string path
     * @returns Object
     */
    jsonFile(path) {
        let content = this.get(path);

        try {
            return JSON.parse(content);
        } catch (e) {
            throw new Error(`Invalid json syntax in ` + path);
        }
    }

    /**
     * Alias to jsonFile
     */
    getJson(path) {
        return this.jsonFile(path);
    }

    /**
     * Get the content of the given file
     * 
     * @param  string path
     * @returns string
     */
    get(path) {
        return fs.readFile(path, 'utf8');
    }

    /**
     * Determine if the given path is directory
     * @see https://stackoverflow.com/a/53530146/3380387
     * 
     * @param   string path
     * @returns boolean
     */
    isDir(path) {
        try {
            return fs.lstat(path).isDirectory();
        } catch (e) {
            // lstat throws an error if path doesn't exist
            return false;
        }
    }

    /**
     * remove the given path
     */
    remove(path) {
        fs.remove(path);
    }

    /**
     * Alias to remove
     */
    unlink(path) {
        return this.remove(path);
    }

    /**
     * Put the given content into the given file path
     * 
     * @param string path 
     * @param string content 
     */
    put(path, content) {
        fs.writeFile(path, content, 'utf8');
    }

    /**
     * Put the given object in the given path in json format
     * 
     * @param  string path
     * @param  object object 
     */
    putJson(path, object) {
        fs.writeJson(path, object, {
            spaces: 4,
        });
    }

    /**
     * Create new directory
     * 
     * @param  string path
     * @param ? mode 
     */
    makeDirectory(path, mode) {
        fs.ensureDir(path, {
            mode,
        });
    }

    /**
     * Move the given source to new target
     * 
     */
    move(source, target) {
        fs.move(source, target);
    }

    /**
     * Rename
     */
    rename(oldPath, newPath) {
        fs.rename(oldPath, newPath);
    }

    /**
     * Determine if the given path is file
     * 
     * @param   string path
     * @returns boolean
     */
    isFile(path) {
        try {
            return fs.lstat(path).isFile();
        } catch (e) {
            return false;
        }
    }

    /**
     * Copy the given source to the new target 
     * 
     * @param   string source
     * @param   string target
     * @returns void
     */
    copy(source, target) {
        return fs.copy(source, target);
    }
}

class FileSystem {
    /**
     * Determine if the given path exists 
     * 
     * @param   string path
     * @returns boolean
     */
    exists(path) {
        return fs.existsSync(path);
    }

    /**
     * Load the given json file path
     * 
     * @param  string path
     * @returns Object
     */
    jsonFile(path) {
        let content = this.get(path);

        try {
            return JSON.parse(content);
        } catch (e) {
            throw new Error(`Invalid json syntax in ` + path);
        }
    }

    /**
     * Alias to jsonFile
     */
    getJson(path) {
        return this.jsonFile(path);
    }

    /**
     * Get the content of the given file
     * 
     * @param  string path
     * @returns string
     */
    get(path) {
        return fs.readFileSync(path, 'utf8');
    }

    /**
     * Determine if the given path is directory
     * @see https://stackoverflow.com/a/53530146/3380387
     * 
     * @param   string path
     * @returns boolean
     */
    isDir(path) {
        try {
            return fs.lstatSync(path).isDirectory();
        } catch (e) {
            // lstatSync throws an error if path doesn't exist
            return false;
        }
    }

    /**
     * remove the given path
     */
    remove(path) {
        fs.removeSync(path);
    }

    /**
     * Alias to remove
     */
    unlink(path) {
        return this.remove(path);
    }

    /**
     * Put the given content into the given file path
     * 
     * @param string path 
     * @param string content 
     */
    put(path, content) {
        fs.writeFileSync(path, content, 'utf8');
    }

    /**
     * Put the given object in the given path in json format
     * 
     * @param  string path
     * @param  object object 
     */
    putJson(path, object, formatted = 4) {
        fs.writeJsonSync(path, object, {
            spaces: formatted // to format the list 
        });
    }

    /**
     * Create new directory
     * 
     * @param  string path
     * @param ? mode 
     */
    makeDirectory(path, mode) {
        fs.ensureDirSync(path, {
            mode,
        });
    }

    /**
     * Move the given source to new target
     * 
     */
    move(source, target) {
        fs.moveSync(source, target);
    }

    /**
     * Rename
     */
    rename(oldPath, newPath) {
        fs.renameSync(oldPath, newPath);
    }

    /**
     * Determine if the given path is file
     * 
     * @param   string path
     * @returns boolean
     */
    isFile(path) {
        try {
            return fs.lstatSync(path).isFile();
        } catch (e) {
            return false;
        }
    }

    /**
     * Copy the given source to the new target 
     * 
     * @param   string source
     * @param   string target
     * @returns void
     */
    copy(source, target) {
        return fs.copySync(source, target);
    }

    /**
     * Get a async version of the fs
     */
    get async() {
        return new AsyncFileSystem;
    }
}

module.exports = new FileSystem;